@@ -4,7 +4,7 @@
  * Copyright (C) 2013-2018 Richard Hughes <richard@hughsie.com>
  * Copyright (C) 2014-2018 Kalev Lember <klember@redhat.com>
  *
- * SPDX-License-Identifier: GPL-2.0+
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #pragma once
@@ -12,10 +12,17 @@
 #include <glib-object.h>
 #include <gdk/gdk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <libsoup/soup.h>
 #include <appstream.h>
 
+#include <gs-app-permissions.h>
+
 G_BEGIN_DECLS
 
+/* Dependency loop means we can’t include the header. */
+typedef struct _GsPlugin GsPlugin;
+typedef struct _GsAppList GsAppList;
+
 #define GS_TYPE_APP (gs_app_get_type ())
 
 G_DECLARE_DERIVABLE_TYPE (GsApp, gs_app, GS, APP, GObject)
@@ -42,6 +49,11 @@ struct _GsAppClass
  * @GS_APP_STATE_UPDATABLE_LIVE:		Application is installed and updatable live
  * @GS_APP_STATE_PURCHASABLE:			Application is available for purchasing
  * @GS_APP_STATE_PURCHASING:			Application is being purchased
+ * @GS_APP_STATE_PENDING_INSTALL:		Application is installed, but may have pending some actions,
+ *						like restart, to finish it
+ * @GS_APP_STATE_PENDING_REMOVE:		Application is removed, but may have pending some actions,
+ *						like restart, to finish it
+ * @GS_APP_STATE_DOWNLOADING:			Application is being downloaded
  *
  * The application state.
  **/
@@ -58,6 +70,9 @@ typedef enum {
 	GS_APP_STATE_UPDATABLE_LIVE,			/* Since: 0.5.4 */
 	GS_APP_STATE_PURCHASABLE,			/* Since: 0.5.17 */
 	GS_APP_STATE_PURCHASING,			/* Since: 0.5.17 */
+	GS_APP_STATE_PENDING_INSTALL,			/* Since: 41 */
+	GS_APP_STATE_PENDING_REMOVE,			/* Since: 41 */
+	GS_APP_STATE_DOWNLOADING,			/* Since: 46 */
 	GS_APP_STATE_LAST  /*< skip >*/
 } GsAppState;
 
@@ -82,14 +97,8 @@ typedef enum {
  * @GS_APP_KUDO_MY_LANGUAGE:		Localised in my language
  * @GS_APP_KUDO_RECENT_RELEASE:		Released recently
  * @GS_APP_KUDO_FEATURED_RECOMMENDED:	Chosen for the front page
- * @GS_APP_KUDO_MODERN_TOOLKIT:		Uses a modern toolkit
- * @GS_APP_KUDO_SEARCH_PROVIDER:	Provides a search provider
- * @GS_APP_KUDO_INSTALLS_USER_DOCS:	Installs user docs
- * @GS_APP_KUDO_USES_NOTIFICATIONS:	Registers notifications
  * @GS_APP_KUDO_HAS_KEYWORDS:		Has at least 1 keyword
  * @GS_APP_KUDO_HAS_SCREENSHOTS:	Supplies screenshots
- * @GS_APP_KUDO_POPULAR:		Is popular
- * @GS_APP_KUDO_HIGH_CONTRAST:		Installs a high contrast icon
  * @GS_APP_KUDO_HI_DPI_ICON:		Installs a HiDPI icon
  * @GS_APP_KUDO_SANDBOXED:		Application is sandboxed
  * @GS_APP_KUDO_SANDBOXED_SECURE:	Application is sandboxed securely
@@ -100,14 +109,8 @@ typedef enum {
 	GS_APP_KUDO_MY_LANGUAGE			= 1 << 0,
 	GS_APP_KUDO_RECENT_RELEASE		= 1 << 1,
 	GS_APP_KUDO_FEATURED_RECOMMENDED	= 1 << 2,
-	GS_APP_KUDO_MODERN_TOOLKIT		= 1 << 3,
-	GS_APP_KUDO_SEARCH_PROVIDER		= 1 << 4,
-	GS_APP_KUDO_INSTALLS_USER_DOCS		= 1 << 5,
-	GS_APP_KUDO_USES_NOTIFICATIONS		= 1 << 6,
 	GS_APP_KUDO_HAS_KEYWORDS		= 1 << 7,
 	GS_APP_KUDO_HAS_SCREENSHOTS		= 1 << 9,
-	GS_APP_KUDO_POPULAR			= 1 << 10,
-	GS_APP_KUDO_HIGH_CONTRAST		= 1 << 13,
 	GS_APP_KUDO_HI_DPI_ICON			= 1 << 14,
 	GS_APP_KUDO_SANDBOXED			= 1 << 15,
 	GS_APP_KUDO_SANDBOXED_SECURE		= 1 << 16,
@@ -119,15 +122,14 @@ typedef enum {
  * @GS_APP_QUIRK_NONE:			No special attributes
  * @GS_APP_QUIRK_PROVENANCE:		Installed by OS vendor
  * @GS_APP_QUIRK_COMPULSORY:		Cannot be removed
- * @GS_APP_QUIRK_HAS_SOURCE:		Has a source to allow staying up-to-date
+ * @GS_APP_QUIRK_LOCAL_HAS_REPOSITORY:	App is from a local file, but it contains repository information which allows it to be kept up-to-date (Since: 49)
  * @GS_APP_QUIRK_IS_WILDCARD:		Matches applications from any plugin
  * @GS_APP_QUIRK_NEEDS_REBOOT:		A reboot is required after the action
  * @GS_APP_QUIRK_NOT_REVIEWABLE:	The app is not reviewable
- * @GS_APP_QUIRK_HAS_SHORTCUT:		The app has a shortcut in the system
  * @GS_APP_QUIRK_NOT_LAUNCHABLE:	The app is not launchable (run-able)
  * @GS_APP_QUIRK_NEEDS_USER_ACTION:	The component requires some kind of user action
  * @GS_APP_QUIRK_IS_PROXY:		Is a proxy app that operates on other applications
- * @GS_APP_QUIRK_REMOVABLE_HARDWARE:	The device is unusable whilst the action is performed
+ * @GS_APP_QUIRK_UNUSABLE_DURING_UPDATE:The device is unusable whilst the action is performed
  * @GS_APP_QUIRK_DEVELOPER_VERIFIED:	The app developer has been verified
  * @GS_APP_QUIRK_PARENTAL_FILTER:	The app has been filtered by parental controls, and should be hidden
  * @GS_APP_QUIRK_NEW_PERMISSIONS:	The update requires new permissions
@@ -135,6 +137,7 @@ typedef enum {
  * @GS_APP_QUIRK_HIDE_FROM_SEARCH:	The app should not be shown in search results
  * @GS_APP_QUIRK_HIDE_EVERYWHERE:	The app should not be shown anywhere (it’s blocklisted)
  * @GS_APP_QUIRK_DO_NOT_AUTO_UPDATE:	The app should not be automatically updated
+ * @GS_APP_QUIRK_FROM_DEVELOPMENT_REPOSITORY: The app is from a development/beta repository (Since: 49)
  *
  * The application attributes.
  **/
@@ -142,15 +145,15 @@ typedef enum {
 	GS_APP_QUIRK_NONE		= 0,		/* Since: 3.32 */
 	GS_APP_QUIRK_PROVENANCE		= 1 << 0,	/* Since: 3.32 */
 	GS_APP_QUIRK_COMPULSORY		= 1 << 1,	/* Since: 3.32 */
-	GS_APP_QUIRK_HAS_SOURCE		= 1 << 2,	/* Since: 3.32 */
+	GS_APP_QUIRK_LOCAL_HAS_REPOSITORY	= 1 << 2,	/* Since: 49 */
 	GS_APP_QUIRK_IS_WILDCARD	= 1 << 3,	/* Since: 3.32 */
 	GS_APP_QUIRK_NEEDS_REBOOT	= 1 << 4,	/* Since: 3.32 */
 	GS_APP_QUIRK_NOT_REVIEWABLE	= 1 << 5,	/* Since: 3.32 */
-	GS_APP_QUIRK_HAS_SHORTCUT	= 1 << 6,	/* Since: 3.32 */
+	/* there’s a hole here where GS_APP_QUIRK_HAS_SHORTCUT used to be */
 	GS_APP_QUIRK_NOT_LAUNCHABLE	= 1 << 7,	/* Since: 3.32 */
 	GS_APP_QUIRK_NEEDS_USER_ACTION	= 1 << 8,	/* Since: 3.32 */
 	GS_APP_QUIRK_IS_PROXY 		= 1 << 9,	/* Since: 3.32 */
-	GS_APP_QUIRK_REMOVABLE_HARDWARE	= 1 << 10,	/* Since: 3.32 */
+	GS_APP_QUIRK_UNUSABLE_DURING_UPDATE	= 1 << 10,	/* Since: 44 */
 	GS_APP_QUIRK_DEVELOPER_VERIFIED	= 1 << 11,	/* Since: 3.32 */
 	GS_APP_QUIRK_PARENTAL_FILTER	= 1 << 12,	/* Since: 3.32 */
 	GS_APP_QUIRK_NEW_PERMISSIONS	= 1 << 13,	/* Since: 3.32 */
@@ -158,12 +161,31 @@ typedef enum {
 	GS_APP_QUIRK_HIDE_FROM_SEARCH	= 1 << 15,	/* Since: 3.32 */
 	GS_APP_QUIRK_HIDE_EVERYWHERE	= 1 << 16,	/* Since: 3.36 */
 	GS_APP_QUIRK_DO_NOT_AUTO_UPDATE	= 1 << 17,	/* Since: 3.36 */
+	GS_APP_QUIRK_FROM_DEVELOPMENT_REPOSITORY = 1 << 18,	/* Since: 49 */
 	GS_APP_QUIRK_LAST  /*< skip >*/
 } GsAppQuirk;
 
 #define	GS_APP_INSTALL_DATE_UNSET		0
 #define	GS_APP_INSTALL_DATE_UNKNOWN		1 /* 1s past the epoch */
-#define	GS_APP_SIZE_UNKNOWABLE			G_MAXUINT64
+
+/**
+ * GsSizeType:
+ * @GS_SIZE_TYPE_UNKNOWN:	Size is unknown
+ * @GS_SIZE_TYPE_UNKNOWABLE:	Size is unknown and is impossible to calculate
+ * @GS_SIZE_TYPE_VALID:		Size is known and valid
+ *
+ * Types of download or file size for applications.
+ *
+ * These are used to represent the validity of properties like
+ * #GsApp:size-download.
+ *
+ * Since: 43
+ */
+typedef enum {
+	GS_SIZE_TYPE_UNKNOWN,
+	GS_SIZE_TYPE_UNKNOWABLE,
+	GS_SIZE_TYPE_VALID,
+} GsSizeType;
 
 /**
  * GsAppQuality:
@@ -182,31 +204,39 @@ typedef enum {
 	GS_APP_QUALITY_LAST  /*< skip >*/
 } GsAppQuality;
 
+/**
+ * GsAppIconsState:
+ * @GS_APP_ICONS_STATE_UNKNOWN:		The state of the icons is unknown
+ * @GS_APP_ICONS_STATE_PENDING_DOWNLOAD:	Icons are in queue to be downloaded
+ * @GS_APP_ICONS_STATE_DOWNLOADING:	Icons are downloading
+ * @GS_APP_ICONS_STATE_AVAILABLE:	Icons are available
+ *
+ * State of the icons of the application.
+ *
+ * Since: 44
+ **/
 typedef enum {
-	GS_APP_PERMISSIONS_UNKNOWN 		= 0,
-	GS_APP_PERMISSIONS_NONE			= 1 << 0,
-	GS_APP_PERMISSIONS_NETWORK 		= 1 << 1,
-	GS_APP_PERMISSIONS_SYSTEM_BUS   	= 1 << 2,
-	GS_APP_PERMISSIONS_SESSION_BUS		= 1 << 3,
-	GS_APP_PERMISSIONS_DEVICES 		= 1 << 4,
-	GS_APP_PERMISSIONS_HOME_FULL 		= 1 << 5,
-	GS_APP_PERMISSIONS_HOME_READ		= 1 << 6,
-	GS_APP_PERMISSIONS_FILESYSTEM_FULL	= 1 << 7,
-	GS_APP_PERMISSIONS_FILESYSTEM_READ	= 1 << 8,
-	GS_APP_PERMISSIONS_DOWNLOADS_FULL 	= 1 << 9,
-	GS_APP_PERMISSIONS_DOWNLOADS_READ	= 1 << 10,
-	GS_APP_PERMISSIONS_SETTINGS		= 1 << 11,
-	GS_APP_PERMISSIONS_X11			= 1 << 12,
-	GS_APP_PERMISSIONS_ESCAPE_SANDBOX	= 1 << 13,
-	GS_APP_PERMISSIONS_LAST  /*< skip >*/
-} GsAppPermissions;
+	GS_APP_ICONS_STATE_UNKNOWN,
+	GS_APP_ICONS_STATE_PENDING_DOWNLOAD,
+	GS_APP_ICONS_STATE_DOWNLOADING,
+	GS_APP_ICONS_STATE_AVAILABLE,
+} GsAppIconsState;
 
-#define LIMITED_PERMISSIONS (GS_APP_PERMISSIONS_SETTINGS | \
-			GS_APP_PERMISSIONS_NETWORK | \
-			GS_APP_PERMISSIONS_DOWNLOADS_READ | \
-			GS_APP_PERMISSIONS_DOWNLOADS_FULL)
-#define MEDIUM_PERMISSIONS (LIMITED_PERMISSIONS | \
-			GS_APP_PERMISSIONS_X11)
+/**
+ * GsColorScheme:
+ * @GS_COLOR_SCHEME_ANY: any color scheme
+ * @GS_COLOR_SCHEME_LIGHT: light color scheme
+ * @GS_COLOR_SCHEME_DARK: dark color scheme
+ *
+ * Define color scheme.
+ *
+ * Since: 47
+ **/
+typedef enum {
+	GS_COLOR_SCHEME_ANY = 0,
+	GS_COLOR_SCHEME_LIGHT = 1,
+	GS_COLOR_SCHEME_DARK = 2
+} GsColorScheme;
 
 /**
  * GS_APP_PROGRESS_UNKNOWN:
@@ -267,13 +297,13 @@ void		 gs_app_set_name		(GsApp		*app,
 const gchar	*gs_app_get_renamed_from	(GsApp		*app);
 void		 gs_app_set_renamed_from	(GsApp		*app,
 						 const gchar	*renamed_from);
-const gchar	*gs_app_get_source_default	(GsApp		*app);
+const gchar	*gs_app_get_default_source	(GsApp		*app);
 void		 gs_app_add_source		(GsApp		*app,
 						 const gchar	*source);
 GPtrArray	*gs_app_get_sources		(GsApp		*app);
 void		 gs_app_set_sources		(GsApp		*app,
 						 GPtrArray	*sources);
-const gchar	*gs_app_get_source_id_default	(GsApp		*app);
+const gchar	*gs_app_get_default_source_id	(GsApp		*app);
 void		 gs_app_add_source_id		(GsApp		*app,
 						 const gchar	*source_id);
 GPtrArray	*gs_app_get_source_ids		(GsApp		*app);
@@ -344,28 +374,35 @@ const gchar	*gs_app_get_update_version	(GsApp		*app);
 const gchar	*gs_app_get_update_version_ui	(GsApp		*app);
 void		 gs_app_set_update_version	(GsApp		*app,
 						 const gchar	*update_version);
-const gchar	*gs_app_get_update_details	(GsApp		*app);
-void		 gs_app_set_update_details	(GsApp		*app,
-						 const gchar	*update_details);
+const gchar	*gs_app_get_update_details_markup
+						(GsApp		*app);
+void		 gs_app_set_update_details_markup
+						(GsApp		*app,
+						 const gchar	*markup);
+void		 gs_app_set_update_details_text	(GsApp		*app,
+						 const gchar	*text);
+gboolean	 gs_app_get_update_details_set	(GsApp		*app);
 AsUrgencyKind	 gs_app_get_update_urgency	(GsApp		*app);
 void		 gs_app_set_update_urgency	(GsApp		*app,
 						 AsUrgencyKind	 update_urgency);
-const gchar	*gs_app_get_management_plugin	(GsApp		*app);
+GsPlugin	*gs_app_dup_management_plugin	(GsApp		*app);
+gboolean	 gs_app_has_management_plugin	(GsApp		*app,
+						 GsPlugin	*plugin);
 void		 gs_app_set_management_plugin	(GsApp		*app,
-						 const gchar	*management_plugin);
+						 GsPlugin	*management_plugin);
 GIcon		*gs_app_get_icon_for_size	(GsApp		*app,
 						 guint		 size,
 						 guint		 scale,
 						 const gchar	*fallback_icon_name);
-GPtrArray	*gs_app_get_icons		(GsApp		*app);
+GPtrArray	*gs_app_dup_icons		(GsApp		*app);
+gboolean	 gs_app_has_icons		(GsApp		*app);
 void		 gs_app_add_icon		(GsApp		*app,
 						 GIcon		*icon);
 void		 gs_app_remove_all_icons	(GsApp		*app);
-gboolean	 gs_app_get_use_drop_shadow	(GsApp		*app);
 GFile		*gs_app_get_local_file		(GsApp		*app);
 void		 gs_app_set_local_file		(GsApp		*app,
 						 GFile		*local_file);
-AsContentRating	*gs_app_get_content_rating	(GsApp		*app);
+AsContentRating	*gs_app_dup_content_rating	(GsApp		*app);
 void		 gs_app_set_content_rating	(GsApp		*app,
 						 AsContentRating *content_rating);
 GsApp		*gs_app_get_runtime		(GsApp		*app);
@@ -398,16 +435,36 @@ AsProvided	*gs_app_get_provided_for_kind	(GsApp		*app,
 void		 gs_app_add_provided_item	(GsApp		*app,
 						 AsProvidedKind kind,
 						 const gchar	*item);
-guint64		 gs_app_get_size_installed	(GsApp		*app);
+GsSizeType	 gs_app_get_size_installed	(GsApp		*app,
+						 guint64	*size_bytes_out);
 void		 gs_app_set_size_installed	(GsApp		*app,
-						 guint64	 size_installed);
-guint64		 gs_app_get_size_download	(GsApp		*app);
+						 GsSizeType	 size_type,
+						 guint64	 size_bytes);
+GsSizeType	 gs_app_get_size_installed_dependencies
+						(GsApp		*app,
+						 guint64	*size_bytes_out);
+GsSizeType	 gs_app_get_size_user_data	(GsApp		*app,
+						 guint64	*size_bytes_out);
+void		 gs_app_set_size_user_data	(GsApp		*app,
+						 GsSizeType	 size_type,
+						 guint64	 size_bytes);
+GsSizeType	 gs_app_get_size_cache_data	(GsApp		*app,
+						 guint64	*size_bytes_out);
+void		 gs_app_set_size_cache_data	(GsApp		*app,
+						 GsSizeType	 size_type,
+						 guint64	 size_bytes);
+GsSizeType	 gs_app_get_size_download	(GsApp		*app,
+						 guint64	*size_bytes_out);
 void		 gs_app_set_size_download	(GsApp		*app,
-						 guint64	 size_download);
+						 GsSizeType	 size_type,
+						 guint64	 size_bytes);
+GsSizeType	 gs_app_get_size_download_dependencies
+						(GsApp		*app,
+						 guint64	*size_bytes_out);
 void		 gs_app_add_related		(GsApp		*app,
 						 GsApp		*app2);
-void		 gs_app_add_addon		(GsApp		*app,
-						 GsApp		*addon);
+void		 gs_app_add_addons		(GsApp		*app,
+						 GsAppList	*addons);
 void		 gs_app_add_history		(GsApp		*app,
 						 GsApp		*app2);
 guint64		 gs_app_get_install_date	(GsApp		*app);
@@ -424,9 +481,15 @@ void		 gs_app_set_key_colors		(GsApp		*app,
 						 GArray		*key_colors);
 void		 gs_app_add_key_color		(GsApp		*app,
 						 GdkRGBA	*key_color);
-void            gs_app_set_is_update_downloaded (GsApp         *app,
-                                                 gboolean       is_update_downloaded);
-gboolean        gs_app_get_is_update_downloaded (GsApp         *app);
+gboolean	 gs_app_get_user_key_colors	(GsApp		*app);
+void		 gs_app_set_key_color_for_color_scheme
+						(GsApp		*app,
+						 GsColorScheme	 for_color_scheme,
+						 const GdkRGBA	*rgba);
+gboolean	 gs_app_get_key_color_for_color_scheme
+						(GsApp		*app,
+						 GsColorScheme	 for_color_scheme,
+						 GdkRGBA	*out_rgba);
 gboolean	 gs_app_has_category		(GsApp		*app,
 						 const gchar	*category);
 void		 gs_app_add_category		(GsApp		*app,
@@ -456,20 +519,46 @@ void		 gs_app_remove_quirk		(GsApp		*app,
 						 GsAppQuirk	 quirk);
 gboolean	 gs_app_is_installed		(GsApp		*app);
 gboolean	 gs_app_is_updatable		(GsApp		*app);
-gchar		*gs_app_get_origin_ui		(GsApp		*app);
+gchar		*gs_app_dup_origin_ui		(GsApp		*app,
+						 gboolean	 with_packaging_format);
 void		 gs_app_set_origin_ui		(GsApp		*app,
 						 const gchar	*origin_ui);
 gchar		*gs_app_get_packaging_format	(GsApp		*app);
+const gchar	*gs_app_get_packaging_format_raw(GsApp *app);
 void		 gs_app_subsume_metadata	(GsApp		*app,
 						 GsApp		*donor);
-GsAppPermissions gs_app_get_permissions		(GsApp		*app);
+GsAppPermissions *
+		 gs_app_dup_permissions		(GsApp		*app);
 void		 gs_app_set_permissions		(GsApp		*app,
-						 GsAppPermissions permissions);
-GsAppPermissions gs_app_get_update_permissions	(GsApp		*app);
+						 GsAppPermissions *permissions);
+GsAppPermissions *
+		 gs_app_dup_update_permissions	(GsApp		*app);
 void		 gs_app_set_update_permissions	(GsApp		*app,
-						 GsAppPermissions update_permissions);
+						 GsAppPermissions *update_permissions);
 GPtrArray	*gs_app_get_version_history	(GsApp		*app);
 void		 gs_app_set_version_history	(GsApp		*app,
 						 GPtrArray	*version_history);
+void		gs_app_ensure_icons_downloaded	(GsApp		*app,
+						 SoupSession	*soup_session,
+						 guint		 maximum_icon_size,
+						 guint		 scale,
+						 GCancellable	*cancellable);
+
+GPtrArray	*gs_app_get_relations		(GsApp		*app);
+void		 gs_app_add_relation		(GsApp		*app,
+						 AsRelation	*relation);
+void		 gs_app_set_relations		(GsApp		*app,
+						 GPtrArray	*relations);
+
+gboolean	 gs_app_get_has_translations	(GsApp		*app);
+void		 gs_app_set_has_translations	(GsApp		*app,
+						 gboolean	 has_translations);
+gboolean	 gs_app_is_downloaded		(GsApp		*app);
+
+GsAppIconsState	 gs_app_get_icons_state		(GsApp		*app);
+gboolean	 gs_app_is_application		(GsApp		*app);
+void		 gs_app_set_mok_key_pending	(GsApp          *app,
+						 gboolean        mok_key_pending);
+gboolean	 gs_app_get_mok_key_pending	(GsApp          *app);
 
 G_END_DECLS
